package sun.nio.fs;

import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.nio.file.attribute.BasicFileAttributes;
import java.io.IOException;
import java.nio.file.attribute.FileTime;
import java.util.Set;
import java.nio.file.attribute.BasicFileAttributeView;

abstract class AbstractBasicFileAttributeView implements BasicFileAttributeView, DynamicFileAttributeView
{
    private static final String SIZE_NAME = "size";
    private static final String CREATION_TIME_NAME = "creationTime";
    private static final String LAST_ACCESS_TIME_NAME = "lastAccessTime";
    private static final String LAST_MODIFIED_TIME_NAME = "lastModifiedTime";
    private static final String FILE_KEY_NAME = "fileKey";
    private static final String IS_DIRECTORY_NAME = "isDirectory";
    private static final String IS_REGULAR_FILE_NAME = "isRegularFile";
    private static final String IS_SYMBOLIC_LINK_NAME = "isSymbolicLink";
    private static final String IS_OTHER_NAME = "isOther";
    static final Set<String> basicAttributeNames;
    
    protected AbstractBasicFileAttributeView() {
    }
    
    @Override
    public String name() {
        return "basic";
    }
    
    @Override
    public void setAttribute(String str, Object o) throws IOException {
        if (str.equals("lastModifiedTime")) {
            this.setTimes((FileTime)o, null, null);
            return;
        }
        if (str.equals("lastAccessTime")) {
            this.setTimes(null, (FileTime)o, null);
            return;
        }
        if (str.equals("creationTime")) {
            this.setTimes(null, null, (FileTime)o);
            return;
        }
        throw new IllegalArgumentException("'" + this.name() + ":" + str + "' not recognized");
    }
    
    final void addRequestedBasicAttributes(BasicFileAttributes basicFileAttributes, AbstractBasicFileAttributeView.AttributesBuilder attributesBuilder) {
        if (attributesBuilder.match("size")) {
            attributesBuilder.add("size", Long.valueOf(basicFileAttributes.size()));
        }
        if (attributesBuilder.match("creationTime")) {
            attributesBuilder.add("creationTime", basicFileAttributes.creationTime());
        }
        if (attributesBuilder.match("lastAccessTime")) {
            attributesBuilder.add("lastAccessTime", basicFileAttributes.lastAccessTime());
        }
        if (attributesBuilder.match("lastModifiedTime")) {
            attributesBuilder.add("lastModifiedTime", basicFileAttributes.lastModifiedTime());
        }
        if (attributesBuilder.match("fileKey")) {
            attributesBuilder.add("fileKey", basicFileAttributes.fileKey());
        }
        if (attributesBuilder.match("isDirectory")) {
            attributesBuilder.add("isDirectory", Boolean.valueOf(basicFileAttributes.isDirectory()));
        }
        if (attributesBuilder.match("isRegularFile")) {
            attributesBuilder.add("isRegularFile", Boolean.valueOf(basicFileAttributes.isRegularFile()));
        }
        if (attributesBuilder.match("isSymbolicLink")) {
            attributesBuilder.add("isSymbolicLink", Boolean.valueOf(basicFileAttributes.isSymbolicLink()));
        }
        if (attributesBuilder.match("isOther")) {
            attributesBuilder.add("isOther", Boolean.valueOf(basicFileAttributes.isOther()));
        }
    }
    
    @Override
    public Map<String, Object> readAttributes(String[] array) throws IOException {
        AbstractBasicFileAttributeView.AttributesBuilder create = AbstractBasicFileAttributeView.AttributesBuilder.create(AbstractBasicFileAttributeView.basicAttributeNames, array);
        this.addRequestedBasicAttributes(this.readAttributes(), create);
        return create.unmodifiableMap();
    }
    
    static {
        basicAttributeNames = Util.newSet(new String[] { "size", "creationTime", "lastAccessTime", "lastModifiedTime", "fileKey", "isDirectory", "isRegularFile", "isSymbolicLink", "isOther" });
    }
    
    static class AttributesBuilder
    {
        private Set<String> names;
        private Map<String, Object> map;
        private boolean copyAll;
        
        private AttributesBuilder(Set<String> set, String[] array) {
            this.names = new HashSet();
            this.map = new HashMap();
            for (String str : array) {
                if (str.equals("*")) {
                    this.copyAll = true;
                }
                else {
                    if (!set.contains(str)) {
                        throw new IllegalArgumentException("'" + str + "' not recognized");
                    }
                    this.names.add(str);
                }
            }
        }
        
        static AbstractBasicFileAttributeView.AttributesBuilder create(Set<String> set, String[] array) {
            return new AbstractBasicFileAttributeView.AttributesBuilder(set, array);
        }
        
        boolean match(String s) {
            return this.copyAll || this.names.contains(s);
        }
        
        void add(String s, Object o) {
            this.map.put(s, o);
        }
        
        Map<String, Object> unmodifiableMap() {
            return Collections.unmodifiableMap(this.map);
        }
    }
}
